import os
from datetime import date
import json

import estuary_client


def get_latest_collection(data_collections):
    collection_names = sorted(data_collections, key=lambda collection: collection['created_at'])
    return collection_names[-1]


def pick_unique(data_collection):
    assert len(data_collection) == 1
    return data_collection[0]


def _configure_estuary(url, api_key):
    configuration = estuary_client.Configuration()
    configuration.api_key['Authorization'] = api_key
    configuration.api_key_prefix['Authorization'] = 'Bearer'
    configuration.host = url

    return configuration


def versioned_data(data):
    return data + "-" + str(date.today())


class VersionedUploads:
    def __init__(self, url='https://api.estuary.tech', api_key=os.getenv('APIKEY')):
        config = _configure_estuary(url, api_key)
        api_client = estuary_client.ApiClient(config)
        self.content_api = estuary_client.ContentApi(api_client)
        self.collections_api = estuary_client.CollectionsApi(api_client)
        self.all_collections = self.collections_api.collections_get()

    def add_with_version(self, data, filename=None):
        if not filename:
            filename = data
        data_collections = self.get_collections(filename)

        if not data_collections:
            return self.upload_data_to_collection(data, filename)

        latest_cid = self.get_latest_cid(data_collections)
        new_cid = self.get_new_cid(data)

        if latest_cid != new_cid:
            return self.upload_data_to_collection(data, filename)

    def get_new_cid(self, data):
        new_data = self._add_content(data)
        new_cid = new_data.cid
        return new_cid

    def get_latest_cid(self, data_collections):
        latest_collection = get_latest_collection(data_collections)
        latest_collection_id = latest_collection['uuid']
        latest_data = self.get_content(latest_collection_id)
        latest_data = pick_unique(latest_data)
        latest_cid = latest_data['cid']
        return latest_cid

    def upload_data_to_collection(self, data, filename):
        name_with_version = versioned_data(filename)
        response = self.create_collection(name=name_with_version, description='Collection for ' + name_with_version)
        collection_id = response.uuid
        return self._add_content(data=data, filename=filename, collection_id=collection_id)

    def get_collections(self, name):
        return [
            {'name': collection.name, 'uuid': collection.uuid, 'created_at': collection.created_at}
            for collection in self.all_collections if collection.name.startswith(name)
        ]

    def create_collection(self, name, description):
        body = {'name': name, 'description': description}
        return self.collections_api.collections_post(body)

    def get_content(self, collection_id):
        # todo: Need the json.loads because return type is str
        response = self.collections_api.collections_coluuid_get(coluuid=collection_id)
        response = response.replace("'", '"').replace("False", "false").replace("True", "true")
        return json.loads(response)

    def _add_content(self, data, filename=None, collection_id=None):
        if not filename:
            filename = data
        if collection_id:
            return self.content_api.content_add_post(data=data, filename=filename, coluuid=collection_id)
        return self.content_api.content_add_post(data=data, filename=filename)



from pestuary import autoretrieveApi


def autoretrieve_create(pub_key, addresses):
    return autoretrieveApi.admin_autoretrieve_init_post(addresses, pub_key)


def autoretrieve_list():
    return autoretrieveApi.admin_autoretrieve_list_get()


def autoretrieve_heartbeat(token):
    return autoretrieveApi.autoretrieve_heartbeat_post(token)



from pestuary import pinningApi


# creates a new pin
def pin_create(cid, name):
    return pinningApi.pinning_pins_post({
        "cid": cid,
        "name": name
    })


# list all pins for this user
def pin_list():
    return pinningApi.pinning_pins_get()




from estuary_client import MainCreateCollectionBody

from pestuary import collectionsApi


def collection_create(name, description=''):
    body = MainCreateCollectionBody(name=name, description=description)
    return collectionsApi.collections_post(body)


def add_content_to_collection(coluuid, content_ids):
    return collectionsApi.collections_coluuid_post(coluuid, content_ids)


def collection_list():
    return collectionsApi.collections_get()


# todo what is recursive and how should it be used?
def collection_list_content(collection_uuid, collection_path='', recursive=False):
    return collectionsApi.collections_coluuid_get(collection_uuid, dir=collection_path)


def collection_commit(collection_uuid):
    return collectionsApi.collections_coluuid_commit_post(collection_uuid)


# lists all contents for this user
def content_list():
    limit = '0'  # seems to be ignored #TODO what is limit
    return collectionsApi.content_stats_get(limit)
    # TODO old version sorted by id, do we need that?



import json

import instantcli

import pestuary
from pestuary import pinning
from pestuary import content
from pestuary import collections
from pestuary import autoretrieve


instantcli.filter_function_name = lambda name: not name.startswith("main") and not name.startswith("_")
instantcli.post_call = lambda result: print(json.dumps(result, default=lambda x: x.to_dict()))


def main():
    instantcli.load_module(pinning)
    instantcli.load_module(content)
    instantcli.load_module(collections)
    instantcli.load_module(autoretrieve)
    instantcli.cli()


if __name__ == "__main__":
    main()



import os
import tempfile

from estuary_client import UtilContentAddIpfsBody

from pestuary import contentApi
from .collections import collection_create


def add_string(buffer, filename, coluuid='', dir=''):
    with tempfile.TemporaryDirectory() as tempdir:
        with open(os.path.join(tempdir, filename), 'w') as fp:
            fp.write(buffer)
            fp.flush()
            fp.seek(0)
            return contentApi.content_add_post(fp.name, coluuid=coluuid, dir=dir)


def _add_file(path, collection_uuid='', root_collection_path=''):
    # get only relevant parts of path for directory inside collection and filename
    # /tmp/mydir/subdir/current-file -> [collection_path: /subdir/, filename: current-file]
    collection_path = ''
    if collection_uuid:
        if not root_collection_path:
            print(f"empty root collection path")
            return
        collection_path = '/' + os.path.relpath(path, start=root_collection_path)
    print("Calling api", path, collection_uuid)
    return contentApi.content_add_post(path, coluuid=collection_uuid, dir=collection_path)


def _add_dir(path, collection_uuid='', root_collection_path=''):
    responses = []
    if len(os.listdir(path)) == 0:
        print(f"empty directory '{path}'")
        return responses

    for entry in os.listdir(path):
        fullpath = os.path.join(path, entry)
        if os.path.isfile(fullpath):
            responses.append(_add_file(fullpath, collection_uuid, root_collection_path))
        else:
            responses += _add_dir(fullpath, collection_uuid, root_collection_path)

    return responses


def content_add(path, create_collection=False):
    print("path", path)
    if os.path.isfile(path):
        return _add_file(path)

    collection_uuid = ''
    collection = {}
    if create_collection:
        # collection_name is last part of dir path
        # /tmp/foo/bar/cool-pictures/ -> collection_name: cool-pictures
        collection_name = os.path.basename(os.path.normpath(path))
        collection = collection_create(collection_name)
        collection_uuid = collection.uuid
    print("collection_uuid", collection_uuid)

    responses = _add_dir(path, collection_uuid=collection_uuid,
                         root_collection_path=path)

    return responses, collection


def content_add_ipfs(ipfs):
    body = UtilContentAddIpfsBody(root=ipfs)
    return contentApi.content_add_ipfs_post(body)



from setuptools import setup

setup(
        name="pestuary",
        version="1.0.10",
        package_dir={'':'src'},
        install_requires=[
                "estuary_client",
                "requests",
                "instantcli"
            ],
        entry_points='''
            [console_scripts]
            pestuary=pestuary_cli:main
        ''',
        long_description_content_type="text/markdown",
)
        



import os
import sys
import estuary_client
from estuary_client.rest import ApiException
from .versioned_uploads import versioned_uploads

# ESTUARY_URL='http://localhost:3004'
ESTUARY_URL = 'https://api.estuary.tech'
ESTUARY_KEY = os.getenv('APIKEY')
SHUTTLES = {}
if not ESTUARY_KEY:
    print("$APIKEY environment variable not set")
    sys.exit(1)

configuration = estuary_client.Configuration()
configuration.api_key['Authorization'] = ESTUARY_KEY
# Uncomment below to setup prefix (e.g. Bearer) for API key, if needed
configuration.api_key_prefix['Authorization'] = 'Bearer'
configuration.host = ESTUARY_URL

# create an instance of the API class
userApi = estuary_client.UserApi(estuary_client.ApiClient(configuration))
adminApi = estuary_client.AdminApi(estuary_client.ApiClient(configuration))
autoretrieveApi = estuary_client.AutoretrieveApi(estuary_client.ApiClient(configuration))
collectionsApi = estuary_client.CollectionsApi(estuary_client.ApiClient(configuration))
contentApi = estuary_client.ContentApi(estuary_client.ApiClient(configuration))
dealsApi = estuary_client.DealsApi(estuary_client.ApiClient(configuration))
metricsApi = estuary_client.MetricsApi(estuary_client.ApiClient(configuration))
minerApi = estuary_client.MinerApi(estuary_client.ApiClient(configuration))
netApi = estuary_client.NetApi(estuary_client.ApiClient(configuration))
peeringApi = estuary_client.PeeringApi(estuary_client.ApiClient(configuration))
peersApi = estuary_client.PeersApi(estuary_client.ApiClient(configuration))
pinningApi = estuary_client.PinningApi(estuary_client.ApiClient(configuration))
publicApi = estuary_client.PublicApi(estuary_client.ApiClient(configuration))

try:
    # Get API keys for a user
    api_response = userApi.user_api_keys_get()
except ApiException as e:
    print("Exception when calling UserApi->user_api_keys_get: %s\n" % e)

